# Sigle letters in upper-case are usually abbreviations of names
Α
Β
Γ
Δ
Ε
Ζ
Η
Θ
Ι
Κ
Λ
Μ
Ν
Ξ
Ο
Π
Ρ
Σ
Τ
Υ
Φ
Χ
Ψ
Ω

# Includes abbreviations for the Greek language compiled from various sources (Greek grammar books, Greek language related web content).
Άθαν
Έγχρ
Έκθ
Έσδ
Έφ
Όμ
Α΄Έσδρ
Α΄Έσδ
Α΄Βασ
Α΄Θεσ
Α΄Ιω
Α΄Κορινθ
Α΄Κορ
Α΄Μακκ
Α΄Μακ
Α΄Πέτρ
Α΄Πέτ
Α΄Παραλ
Α΄Πε
Α΄Σαμ
Α΄Τιμ
Α΄Χρον
Α΄Χρ
Α.Β.Α
Α.Β
Α.Ε
Α.Κ.Τ.Ο
Αέθλ
Αέτ
Αίλ.Δ
Αίλ.Τακτ
Αίσ
Αββακ
Αβυδ
Αβ
Αγάκλ
Αγάπ
Αγάπ.Αμαρτ.Σ
Αγάπ.Γεωπ
Αγαθάγγ
Αγαθήμ
Αγαθιν
Αγαθοκλ
Αγαθρχ
Αγαθ
Αγαθ.Ιστ
Αγαλλ
Αγαπητ
Αγγ
Αγησ
Αγλ
Αγορ.Κ
Αγρο.Κωδ
Αγρ.Εξ
Αγρ.Κ
Αγ.Γρ
Αδριαν
Αδρ
Αετ
Αθάν
Αθήν
Αθήν.Επιγρ
Αθήν.Επιτ
Αθήν.Ιατρ
Αθήν.Μηχ
Αθανάσ
Αθαν
Αθηνί
Αθηναγ
Αθηνόδ
Αθ
Αθ.Αρχ
Αιλ
Αιλ.Επιστ
Αιλ.ΖΙ
Αιλ.ΠΙ
Αιλ.απ
Αιμιλ
Αιν.Γαζ
Αιν.Τακτ
Αισχίν
Αισχίν.Επιστ
Αισχ
Αισχ.Αγαμ
Αισχ.Αγ
Αισχ.Αλ
Αισχ.Ελεγ
Αισχ.Επτ.Θ
Αισχ.Ευμ
Αισχ.Ικέτ
Αισχ.Ικ
Αισχ.Περσ
Αισχ.Προμ.Δεσμ
Αισχ.Πρ
Αισχ.Χοηφ
Αισχ.Χο
Αισχ.απ
ΑιτΕ
Αιτ
Αλκ
Αλχιας
Αμ.Π.Ο
Αμβ
Αμμών
Αμ.
Αν.Πειθ.Συμβ.Δικ
Ανακρ
Ανακ
Αναμν.Τόμ
Αναπλ
Ανδ
Ανθλγος
Ανθστης
Αντισθ
Ανχης
Αν
Αποκ
Απρ
Απόδ
Απόφ
Απόφ.Νομ
Απ
Απ.Δαπ
Απ.Διατ
Απ.Επιστ
Αριθ
Αριστοτ
Αριστοφ
Αριστοφ.Όρν
Αριστοφ.Αχ
Αριστοφ.Βάτρ
Αριστοφ.Ειρ
Αριστοφ.Εκκλ
Αριστοφ.Θεσμ
Αριστοφ.Ιππ
Αριστοφ.Λυσ
Αριστοφ.Νεφ
Αριστοφ.Πλ
Αριστοφ.Σφ
Αριστ
Αριστ.Αθ.Πολ
Αριστ.Αισθ
Αριστ.Αν.Πρ
Αριστ.Ζ.Ι
Αριστ.Ηθ.Ευδ
Αριστ.Ηθ.Νικ
Αριστ.Κατ
Αριστ.Μετ
Αριστ.Πολ
Αριστ.Φυσιογν
Αριστ.Φυσ
Αριστ.Ψυχ
Αριστ.Ρητ
Αρμεν
Αρμ
Αρχ.Εκ.Καν.Δ
Αρχ.Ευβ.Μελ
Αρχ.Ιδ.Δ
Αρχ.Νομ
Αρχ.Ν
Αρχ.Π.Ε
Αρ
Αρ.Φορ.Μητρ
Ασμ
Ασμ.ασμ
Αστ.Δ
Αστ.Χρον
Ασ
Ατομ.Γνωμ
Αυγ
Αφρ
Αχ.Νομ
Α
Α.Εγχ.Π
Α.Κ.΄Υδρας
Β΄Έσδρ
Β΄Έσδ
Β΄Βασ
Β΄Θεσ
Β΄Ιω
Β΄Κορινθ
Β΄Κορ
Β΄Μακκ
Β΄Μακ
Β΄Πέτρ
Β΄Πέτ
Β΄Πέ
Β΄Παραλ
Β΄Σαμ
Β΄Τιμ
Β΄Χρον
Β΄Χρ
Β.Ι.Π.Ε
Β.Κ.Τ
Β.Κ.Ψ.Β
Β.Μ
Β.Ο.Α.Κ
Β.Ο.Α
Β.Ο.Δ
Βίβλ
Βαρ
ΒεΘ
Βι.Περ
Βιπερ
Βιργ
Βλγ
Βούλ
Βρ
Γ΄Βασ
Γ΄Μακκ
ΓΕΝμλ
Γέν
Γαλ
Γεν
Γλ
Γν.Ν.Σ.Κρ
Γνωμ
Γν
Γράμμ
Γρηγ.Ναζ
Γρηγ.Νύσ
Γ Νοσ
Γ' Ογκολ
Γ.Ν
Δ΄Βασ
Δ.Β
Δ.Δίκη
Δ.Δίκ
Δ.Ε.Σ
Δ.Ε.Φ.Α
Δ.Ε.Φ
Δ.Εργ.Ν
Δαμ
Δαμ.μνημ.έργ
Δαν
Δασ.Κ
Δεκ
Δελτ.Δικ.Ε.Τ.Ε
Δελτ.Νομ
Δελτ.Συνδ.Α.Ε
Δερμ
Δευτ
Δεύτ
Δημοσθ
Δημόκρ
Δι.Δικ
Διάτ
Διαιτ.Απ
Διαιτ
Διαρκ.Στρατ
Δικ
Διοίκ.Πρωτ
ΔιοικΔνη
Διοικ.Εφ
Διον.Αρ
Διόρθ.Λαθ
Δ.κ.Π
Δνη
Δν
Δογμ.Όρος
Δρ
Δ.τ.Α
Δτ
ΔωδΝομ
Δ.Περ
Δ.Στρ
ΕΔΠολ
ΕΕυρΚ
ΕΙΣ
ΕΝαυτΔ
ΕΣΑμΕΑ
ΕΣΘ
ΕΣυγκΔ
ΕΤρΑξΧρΔ
Ε.Φ.Ε.Τ
Ε.Φ.Ι
Ε.Φ.Ο.Επ.Α
Εβδ
Εβρ
Εγκύκλ.Επιστ
Εγκ
Εε.Αιγ
Εθν.Κ.Τ
Εθν
Ειδ.Δικ.Αγ.Κακ
Εικ
Ειρ.Αθ
Ειρην.Αθ
Ειρην
Έλεγχ
Ειρ
Εισ.Α.Π
Εισ.Ε
Εισ.Ν.Α.Κ
Εισ.Ν.Κ.Πολ.Δ
Εισ.Πρωτ
Εισηγ.Έκθ
Εισ
Εκκλ
Εκκ
Εκ
Ελλ.Δνη
Εν.Ε
Εξ
Επ.Αν
Επ.Εργ.Δ
Επ.Εφ
Επ.Κυπ.Δ
Επ.Μεσ.Αρχ
Επ.Νομ
Επίκτ
Επίκ
Επι.Δ.Ε
Επιθ.Ναυτ.Δικ
Επικ
Επισκ.Ε.Δ
Επισκ.Εμπ.Δικ
Επιστ.Επετ.Αρμ
Επιστ.Επετ
Επιστ.Ιερ
Επιτρ.Προστ.Συνδ.Στελ
Επιφάν
Επτ.Εφ
Επ.Ιρ
Επ.Ι
Εργ.Ασφ.Νομ
Ερμ.Α.Κ
Ερμη.Σ
Εσθ
Εσπερ
Ετρ.Δ
Ευκλ
Ευρ.Δ.Δ.Α
Ευρ.Σ.Δ.Α
Ευρ.ΣτΕ
Ευρατόμ
Ευρ.Άλκ
Ευρ.Ανδρομ
Ευρ.Βάκχ
Ευρ.Εκ
Ευρ.Ελ
Ευρ.Ηλ
Ευρ.Ηρακ
Ευρ.Ηρ
Ευρ.Ηρ.Μαιν
Ευρ.Ικέτ
Ευρ.Ιππόλ
Ευρ.Ιφ.Α
Ευρ.Ιφ.Τ
Ευρ.Ι.Τ
Ευρ.Κύκλ
Ευρ.Μήδ
Ευρ.Ορ
Ευρ.Ρήσ
Ευρ.Τρωάδ
Ευρ.Φοίν
Εφ.Αθ
Εφ.Εν
Εφ.Επ
Εφ.Θρ
Εφ.Θ
Εφ.Ι
Εφ.Κερ
Εφ.Κρ
Εφ.Λ
Εφ.Ν
Εφ.Πατ
Εφ.Πειρ
Εφαρμ.Δ.Δ
Εφαρμ
Εφεσ
Εφημ
Εφ
Ζαχ
Ζιγ
Ζυ
Ζχ
ΗΕ.Δ
Ημερ
Ηράκλ
Ηροδ
Ησίοδ
Ησ
Η.Ε.Γ
ΘΗΣ
ΘΡ
Θαλ
Θεοδ
Θεοφ
Θεσ
Θεόδ.Μοψ
Θεόκρ
Θεόφιλ
Θουκ
Θρ
Θρ.Ε
Θρ.Ιερ
Θρ.Ιρ
Ιακ
Ιαν
Ιβ
Ιδθ
Ιδ
Ιεζ
Ιερ
Ιζ
Ιησ
Ιησ.Ν
Ικ
Ιλ
Ιν
Ιουδ
Ιουστ
Ιούδα
Ιούλ
Ιούν
Ιπποκρ
Ιππόλ
Ιρ
Ισίδ.Πηλ
Ισοκρ
Ισ.Ν
Ιωβ
Ιωλ
Ιων
Ιω
ΚΟΣ
ΚΟ.ΜΕ.ΚΟΝ
ΚΠοινΔ
ΚΠολΔ
ΚαΒ
Καλ
Καλ.Τέχν
ΚανΒ
Καν.Διαδ
Κατάργ
Κλ
ΚοινΔ
Κολσ
Κολ
Κον
Κορ
Κος
ΚριτΕπιθ
ΚριτΕ
Κριτ
Κρ
ΚτΒ
ΚτΕ
ΚτΠ
Κυβ
Κυπρ
Κύριλ.Αλεξ
Κύριλ.Ιερ
Λεβ
Λεξ.Σουίδα
Λευϊτ
Λευ
Λκ
Λογ
ΛουκΑμ
Λουκιαν
Λουκ.Έρωτ
Λουκ.Ενάλ.Διάλ
Λουκ.Ερμ
Λουκ.Εταιρ.Διάλ
Λουκ.Ε.Δ
Λουκ.Θε.Δ
Λουκ.Ικ.
Λουκ.Ιππ
Λουκ.Λεξιφ
Λουκ.Μεν
Λουκ.Μισθ.Συν
Λουκ.Ορχ
Λουκ.Περ
Λουκ.Συρ
Λουκ.Τοξ
Λουκ.Τυρ
Λουκ.Φιλοψ
Λουκ.Φιλ
Λουκ.Χάρ
Λουκ.
Λουκ.Αλ
Λοχ
Λυδ
Λυκ
Λυσ
Λωζ
Λ1
Λ2
ΜΟΕφ
Μάρκ
Μέν
Μαλ
Ματθ
Μα
Μιχ
Μκ
Μλ
Μμ
Μον.Δ.Π
Μον.Πρωτ
Μον
Μρ
Μτ
Μχ
Μ.Βασ
Μ.Πλ
ΝΑ
Ναυτ.Χρον
Να
Νδικ
Νεεμ
Νε
Νικ
ΝκΦ
Νμ
ΝοΒ
Νομ.Δελτ.Τρ.Ελ
Νομ.Δελτ
Νομ.Σ.Κ
Νομ.Χρ
Νομ
Νομ.Διεύθ
Νοσ
Ντ
Νόσων
Ν1
Ν2
Ν3
Ν4
Νtot
Ξενοφ
Ξεν
Ξεν.Ανάβ
Ξεν.Απολ
Ξεν.Απομν
Ξεν.Απομ
Ξεν.Ελλ
Ξεν.Ιέρ
Ξεν.Ιππαρχ
Ξεν.Ιππ
Ξεν.Κυρ.Αν
Ξεν.Κύρ.Παιδ
Ξεν.Κ.Π
Ξεν.Λακ.Πολ
Ξεν.Οικ
Ξεν.Προσ
Ξεν.Συμπόσ
Ξεν.Συμπ
Ο΄
Οβδ
Οβ
ΟικΕ
Οικ
Οικ.Πατρ
Οικ.Σύν.Βατ
Ολομ
Ολ
Ολ.Α.Π
Ομ.Ιλ
Ομ.Οδ
ΟπΤοιχ
Οράτ
Ορθ
ΠΡΟ.ΠΟ
Πίνδ
Πίνδ.Ι
Πίνδ.Νεμ
Πίνδ.Ν
Πίνδ.Ολ
Πίνδ.Παθ
Πίνδ.Πυθ
Πίνδ.Π
ΠαγΝμλγ
Παν
Παρμ
Παροιμ
Παρ
Παυσ
Πειθ.Συμβ
ΠειρΝ
Πελ
ΠεντΣτρ
Πεντ
Πεντ.Εφ
ΠερΔικ
Περ.Γεν.Νοσ
Πετ
Πλάτ
Πλάτ.Αλκ
Πλάτ.Αντ
Πλάτ.Αξίοχ
Πλάτ.Απόλ
Πλάτ.Γοργ
Πλάτ.Ευθ
Πλάτ.Θεαίτ
Πλάτ.Κρατ
Πλάτ.Κριτ
Πλάτ.Λύσ
Πλάτ.Μεν
Πλάτ.Νόμ
Πλάτ.Πολιτ
Πλάτ.Πολ
Πλάτ.Πρωτ
Πλάτ.Σοφ.
Πλάτ.Συμπ
Πλάτ.Τίμ
Πλάτ.Φαίδρ
Πλάτ.Φιλ
Πλημ
Πλούτ
Πλούτ.Άρατ
Πλούτ.Αιμ
Πλούτ.Αλέξ
Πλούτ.Αλκ
Πλούτ.Αντ
Πλούτ.Αρτ
Πλούτ.Ηθ
Πλούτ.Θεμ
Πλούτ.Κάμ
Πλούτ.Καίσ
Πλούτ.Κικ
Πλούτ.Κράσ
Πλούτ.Κ
Πλούτ.Λυκ
Πλούτ.Μάρκ
Πλούτ.Μάρ
Πλούτ.Περ
Πλούτ.Ρωμ
Πλούτ.Σύλλ
Πλούτ.Φλαμ
Πλ
Ποιν.Δικ
Ποιν.Δ
Ποιν.Ν
Ποιν.Χρον
Ποιν.Χρ
Πολ.Δ
Πολ.Πρωτ
Πολ
Πολ.Μηχ
Πολ.Μ
Πρακτ.Αναθ
Πρακτ.Ολ
Πραξ
Πρμ
Πρξ
Πρωτ
Πρ
Πρ.Αν
Πρ.Λογ
Πταισμ
Πυρ.Καλ
Πόλη
Π.Δ
Π.Δ.Άσμ
ΡΜ.Ε
Ρθ
Ρμ
Ρωμ
ΣΠλημ
Σαπφ
Σειρ
Σολ
Σοφ
Σοφ.Αντιγ
Σοφ.Αντ
Σοφ.Αποσ
Σοφ.Απ
Σοφ.Ηλέκ
Σοφ.Ηλ
Σοφ.Οιδ.Κολ
Σοφ.Οιδ.Τύρ
Σοφ.Ο.Τ
Σοφ.Σειρ
Σοφ.Σολ
Σοφ.Τραχ
Σοφ.Φιλοκτ
Σρ
Σ.τ.Ε
Σ.τ.Π
Στρ.Π.Κ
Στ.Ευρ
Συζήτ
Συλλ.Νομολ
Συλ.Νομ
ΣυμβΕπιθ
Συμπ.Ν
Συνθ.Αμ
Συνθ.Ε.Ε
Συνθ.Ε.Κ
Συνθ.Ν
Σφν
Σφ
Σφ.Σλ
Σχ.Πολ.Δ
Σχ.Συντ.Ε
Σωσ
Σύντ
Σ.Πληρ
ΤΘ
ΤΣ.Δ
Τίτ
Τβ
Τελ.Ενημ
Τελ.Κ
Τερτυλ
Τιμ
Τοπ.Α
Τρ.Ο
Τριμ
Τριμ.Πλ
Τρ.Πλημ
Τρ.Π.Δ
Τ.τ.Ε
Ττ
Τωβ
Υγ
Υπερ
Υπ
Υ.Γ
Φιλήμ
Φιλιπ
Φιλ
Φλμ
Φλ
Φορ.Β
Φορ.Δ.Ε
Φορ.Δνη
Φορ.Δ
Φορ.Επ
Φώτ
Χρ.Ι.Δ
Χρ.Ιδ.Δ
Χρ.Ο
Χρυσ
Ψήφ
Ψαλμ
Ψαλ
Ψλ
Ωριγ
Ωσ
Ω.Ρ.Λ
άγν
άγν.ετυμολ
άγ
άκλ
άνθρ
άπ
άρθρ
άρν
άρ
άτ
άψ
ά
έκδ
έκφρ
έμψ
ένθ.αν
έτ
έ.α
ίδ
αβεστ
αβησσ
αγγλ
αγγ
αδημ
αεροναυτ
αερον
αεροπ
αθλητ
αθλ
αθροιστ
αιγυπτ
αιγ
αιτιολ
αιτ
αι
ακαδ
ακκαδ
αλβ
αλλ
αλφαβητ
αμα
αμερικ
αμερ
αμετάβ
αμτβ
αμφιβ
αμφισβ
αμφ
αμ
ανάλ
ανάπτ
ανάτ
αναβ
αναδαν
αναδιπλασ
αναδιπλ
αναδρ
αναλ
αναν
ανασυλλ
ανατολ
ανατομ
ανατυπ
ανατ
αναφορ
αναφ
ανα.ε
ανδρων
ανθρωπολ
ανθρωπ
ανθ
ανομ
αντίτ
αντδ
αντιγρ
αντιθ
αντικ
αντιμετάθ
αντων
αντ
ανωτ
ανόργ
ανών
αορ
απαρέμφ
απαρφ
απαρχ
απαρ
απλολ
απλοπ
αποβ
αποηχηροπ
αποθ
αποκρυφ
αποφ
απρμφ
απρφ
απρόσ
απόδ
απόλ
απόσπ
απόφ
αραβοτουρκ
αραβ
αραμ
αρβαν
αργκ
αριθμτ
αριθμ
αριθ
αρκτικόλ
αρκ
αρμεν
αρμ
αρνητ
αρσ
αρχαιολ
αρχιτεκτ
αρχιτ
αρχκ
αρχ
αρωμουν
αρωμ
αρ
αρ.μετρ
αρ.φ
ασσυρ
αστρολ
αστροναυτ
αστρον
αττ
αυστραλ
αυτοπ
αυτ
αφγαν
αφηρ
αφομ
αφρικ
αχώρ
αόρ
α.α
α/α
α0
βαθμ
βαθ
βαπτ
βασκ
βεβαιωτ
βεβ
βεδ
βενετ
βεν
βερβερ
βιβλγρ
βιολ
βιομ
βιοχημ
βιοχ
βλάχ
βλ
βλ.λ
βοταν
βοτ
βουλγαρ
βουλγ
βούλ
βραζιλ
βρετον
βόρ
γαλλ
γενικότ
γενοβ
γεν
γερμαν
γερμ
γεωγρ
γεωλ
γεωμετρ
γεωμ
γεωπ
γεωργ
γλυπτ
γλωσσολ
γλωσσ
γλ
γνμδ
γνμ
γνωμ
γοτθ
γραμμ
γραμ
γρμ
γρ
γυμν
δίδες
δίκ
δίφθ
δαν
δεικτ
δεκατ
δηλ
δημογρ
δημοτ
δημώδ
δημ
διάγρ
διάκρ
διάλεξ
διάλ
διάσπ
διαλεκτ
διατρ
διαφ
διαχ
διδα
διεθν
διεθ
δικον
διστ
δισύλλ
δισ
διφθογγοπ
δογμ
δολ
δοτ
δρμ
δρχ
δρ(α)
δωρ
δ
εβρ
εγκλπ
εδ
εθνολ
εθν
ειδικότ
ειδ
ειδ.β
εικ
ειρ
εισ
εκατοστμ
εκατοστ
εκατστ.2
εκατστ.3
εκατ
εκδ
εκκλησ
εκκλ
εκ
ελλην
ελλ
ελνστ
ελπ
εμβ
εμφ
εναλλ
ενδ
ενεργ
ενεστ
ενικ
ενν
εν
εξέλ
εξακολ
εξομάλ
εξ
εο
επέκτ
επίδρ
επίθ
επίρρ
επίσ
επαγγελμ
επανάλ
επανέκδ
επιθ
επικ
επιμ
επιρρ
επιστ
επιτατ
επιφ
επών
επ
εργ
ερμ
ερρινοπ
ερωτ
ετρουσκ
ετυμ
ετ
ευφ
ευχετ
εφ
εύχρ
ε.α
ε/υ
ε0
ζωγρ
ζωολ
ηθικ
ηθ
ηλεκτρολ
ηλεκτρον
ηλεκτρ
ημίτ
ημίφ
ημιφ
ηχηροπ
ηχηρ
ηχομιμ
ηχ
η
θέατρ
θεολ
θετ
θηλ
θρακ
θρησκειολ
θρησκ
θ
ιαπων
ιατρ
ιδιωμ
ιδ
ινδ
ιραν
ισπαν
ιστορ
ιστ
ισχυροπ
ιταλ
ιχθυολ
ιων
κάτ
καθ
κακοσ
καν
καρ
κατάλ
κατατ
κατωτ
κατ
κα
κελτ
κεφ
κινεζ
κινημ
κλητ
κλιτ
κλπ
κλ
κν
κοινωνιολ
κοινων
κοπτ
κουτσοβλαχ
κουτσοβλ
κπ
κρ.γν
κτγ
κτην
κτητ
κτλ
κτ
κυριολ
κυρ
κύρ
κ
κ.ά
κ.ά.π
κ.α
κ.εξ
κ.επ
κ.ε
κ.λπ
κ.λ.π
κ.ού.κ
κ.ο.κ
κ.τ.λ
κ.τ.τ
κ.τ.ό
λέξ
λαογρ
λαπ
λατιν
λατ
λαϊκότρ
λαϊκ
λετ
λιθ
λογιστ
λογοτ
λογ
λουβ
λυδ
λόγ
λ
λ.χ
μέλλ
μέσ
μαθημ
μαθ
μαιευτ
μαλαισ
μαλτ
μαμμων
μεγεθ
μεε
μειωτ
μελ
μεξ
μεσν
μεσογ
μεσοπαθ
μεσοφ
μετάθ
μεταβτ
μεταβ
μετακ
μεταπλ
μεταπτωτ
μεταρ
μεταφορ
μετβ
μετεπιθ
μετεπιρρ
μετεωρολ
μετεωρ
μετον
μετουσ
μετοχ
μετρ
μετ
μητρων
μηχανολ
μηχ
μικροβιολ
μογγολ
μορφολ
μουσ
μπενελούξ
μσνλατ
μσν
μτβ
μτγν
μτγ
μτφρδ
μτφρ
μτφ
μτχ
μυθ
μυκην
μυκ
μφ
μ
μ.ε
μ.μ
μ.π.ε
μ.π.π
μ0
ναυτ
νεοελλ
νεολατιν
νεολατ
νεολ
νεότ
νλατ
νομ
νορβ
νοσ
νότ
ν
ξ.λ
οικοδ
οικολ
οικον
οικ
ολλανδ
ολλ
ομηρ
ομόρρ
ονομ
ον
οπτ
ορθογρ
ορθ
οριστ
ορυκτολ
ορυκτ
ορ
οσετ
οσκ
ουαλ
ουγγρ
ουδ
ουσιαστικοπ
ουσιαστ
ουσ
πίν
παθητ
παθολ
παθ
παιδ
παλαιοντ
παλαιότ
παλ
παππων
παράγρ
παράγ
παράλλ
παράλ
παραγ
παρακ
παραλ
παραπ
παρατ
παρβ
παρετυμ
παροξ
παρων
παρωχ
παρ
παρ.φρ
πατριδων
πατρων
πβ
περιθ
περιλ
περιφρ
περσ
περ
πιθ
πληθ
πληροφ
ποδ
ποιητ
πολιτ
πολλαπλ
πολ
πορτογαλ
πορτ
ποσ
πρακριτ
πρβλ
πρβ
πργ
πρκμ
πρκ
πρλ
προέλ
προβηγκ
προελλ
προηγ
προθεμ
προπαραλ
προπαροξ
προπερισπ
προσαρμ
προσηγορ
προσταχτ
προστ
προσφών
προσ
προτακτ
προτ.Εισ
προφ
προχωρ
πρτ
πρόθ
πρόσθ
πρόσ
πρότ
πρ
πρ.Εφ
πτ
πυ
π
π.Χ
π.μ
π.χ
ρήμ
ρίζ
ρηματ
ρητορ
ριν
ρουμ
ρωμ
ρωσ
ρ
σανσκρ
σαξ
σελ
σερβοκρ
σερβ
σημασιολ
σημδ
σημειολ
σημερ
σημιτ
σημ
σκανδ
σκυθ
σκωπτ
σλαβ
σλοβ
σουηδ
σουμερ
σουπ
σπάν
σπανιότ
σπ
σσ
στατ
στερ
στιγμ
στιχ
στρέμ
στρατιωτ
στρατ
στ
συγγ
συγκρ
συγκ
συμπερ
συμπλεκτ
συμπλ
συμπροφ
συμφυρ
συμφ
συνήθ
συνίζ
συναίρ
συναισθ
συνδετ
συνδ
συνεκδ
συνηρ
συνθετ
συνθ
συνοπτ
συντελ
συντομογρ
συντ
συν
συρ
σχημ
σχ
σύγκρ
σύμπλ
σύμφ
σύνδ
σύνθ
σύντμ
σύντ
σ
σ.π
σ/β
τακτ
τελ
τετρ
τετρ.μ
τεχνλ
τεχνολ
τεχν
τεύχ
τηλεπικ
τηλεόρ
τιμ
τιμ.τομ
τοΣ
τον
τοπογρ
τοπων
τοπ
τοσκ
τουρκ
τοχ
τριτοπρόσ
τροποπ
τροπ
τσεχ
τσιγγ
ττ
τυπ
τόμ
τόνν
τ
τ.μ
τ.χλμ
υβρ
υπερθ
υπερσ
υπερ
υπεύθ
υποθ
υποκορ
υποκ
υποσημ
υποτ
υποφ
υποχωρ
υπόλ
υπόχρ
υπ
υστλατ
υψόμ
υψ
φάκ
φαρμακολ
φαρμ
φιλολ
φιλοσ
φιλοτ
φινλ
φοινικ
φράγκ
φρανκον
φριζ
φρ
φυλλ
φυσιολ
φυσ
φωνηεντ
φωνητ
φωνολ
φων
φωτογρ
φ
φ.τ.μ
χαμιτ
χαρτόσ
χαρτ
χασμ
χαϊδ
χγφ
χειλ
χεττ
χημ
χιλ
χλγρ
χλγ
χλμ
χλμ.2
χλμ.3
χλσγρ
χλστγρ
χλστμ
χλστμ.2
χλστμ.3
χλ
χργρ
χρημ
χρον
χρ
χφ
χ.ε
χ.κ
χ.ο
χ.σ
χ.τ
χ.χ
ψευδ
ψυχαν
ψυχιατρ
ψυχολ
ψυχ
ωκεαν
όμ
όν
όπ.παρ
όπ.π
ό.π
ύψ
1Βσ
1Εσ
1Θσ
1Ιν
1Κρ
1Μκ
1Πρ
1Πτ
1Τμ
2Βσ
2Εσ
2Θσ
2Ιν
2Κρ
2Μκ
2Πρ
2Πτ
2Τμ
3Βσ
3Ιν
3Μκ
4Βσ
